@@ -1,9 +1,9 @@
 /*****
 *
-* Copyright (C) 2000, 2002 Yoann Vandoorselaere <yoann@prelude-ids.org>
-* All Rights Reserved
+* Copyright (C) 2000-2005 PreludeIDS Technologies. All Rights Reserved.
+* Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
-* This file is part of the Prelude program.
+* This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
@@ -26,15 +26,20 @@
 
 typedef struct config config_t;
 
-int config_get_section(config_t *cfg, const char *section, int *line);
+int config_get_next(config_t *cfg, char **section,
+                    char **entry, char **value, unsigned int *line);
 
-const char *config_get(config_t *cfg, const char *section, const char *entry, int *line);
+int config_get_section(config_t *cfg, const char *section, unsigned int *line);
 
-int config_set(config_t *cfg, const char *section, const char *entry, const char *val);
+char *config_get(config_t *cfg, const char *section, const char *entry, unsigned int *line);
 
-config_t *config_open(const char *filename);
+int config_set(config_t *cfg, const char *section, const char *entry, const char *val, unsigned int *line);
+
+int config_open(config_t **ret, const char *filename);
 
 int config_close(config_t *cfg);
 
+int config_del(config_t *cfg, const char *section, const char *entry);
+
 #endif /* _LIBPRELUDE_CONFIG_ENGINE_H */
 
